import {MarkdownConfiguration} from './types/markdown';

const defaultConfiguration: MarkdownConfiguration = {
    html_input: 'ALLOW',
    allow_unsafe_links: false,
    max_nesting_level: Infinity,
    renderer: {
        block_separator: "\n",
        inner_separator: "\n",
        soft_break: "\n"
    },
    slug_normalizer: {
        max_length: 255,
        unique: 'PER_DOCUMENT'
    }
};

export class Markdown {

    private raw: string = '';
    private parsed: string = '';
    public _env: MarkdownConfiguration;

    constructor(option?: MarkdownConfiguration) {
        this._env = {...defaultConfiguration, ...option};
    }

    convert(_value: string): string {

        this.raw = _value;

        // var globals = {
        //     gHtmlBlocks:     [],
        //     gHtmlMdBlocks:   [],
        //     gHtmlSpans:      [],
        //     gUrls:           {},
        //     gTitles:         {},
        //     gDimensions:     {},
        //     gListLevel:      0,
        //     hashLinkCounts:  {},
        //     langExtensions:  langExtensions,
        //     outputModifiers: outputModifiers,
        //     converter:       this,
        //     ghCodeBlocks:    [],
        //     metadata: {
        //         parsed: {},
        //         raw: '',
        //         format: ''
        //     }
        // };

        this.parsed = Markdown.prepareForParse(this.raw);

        // detab
        // text = showdown.subParser('detab')(text, options, globals);
        //
        // /**
        //  * Strip any lines consisting only of spaces and tabs.
        //  * This makes subsequent regexs easier to write, because we can
        //  * match consecutive blank lines with /\n+/ instead of something
        //  * contorted like /[ \t]*\n+/
        //  */
        // text = text.replace(/^[ \t]+$/mg, '');
        //
        // //run languageExtensions
        // showdown.helper.forEach(langExtensions, function (ext) {
        //     text = showdown.subParser('runExtension')(ext, text, options, globals);
        // });
        //
        // // run the sub parsers
        // text = showdown.subParser('metadata')(text, options, globals);
        // text = showdown.subParser('hashPreCodeTags')(text, options, globals);
        // text = showdown.subParser('githubCodeBlocks')(text, options, globals);
        // text = showdown.subParser('hashHTMLBlocks')(text, options, globals);
        // text = showdown.subParser('hashCodeTags')(text, options, globals);
        // text = showdown.subParser('stripLinkDefinitions')(text, options, globals);
        // text = showdown.subParser('blockGamut')(text, options, globals);
        // text = showdown.subParser('unhashHTMLSpans')(text, options, globals);
        // text = showdown.subParser('unescapeSpecialChars')(text, options, globals);
        //
        // // attacklab: Restore dollar signs
        // text = text.replace(/¨D/g, '$$');
        //
        // // attacklab: Restore tremas
        // text = text.replace(/¨T/g, '¨');
        //
        // // render a complete html document instead of a partial if the option is enabled
        // text = showdown.subParser('completeHTMLDocument')(text, options, globals);
        //
        // // Run output modifiers
        // showdown.helper.forEach(outputModifiers, function (ext) {
        //     text = showdown.subParser('runExtension')(ext, text, options, globals);
        // });
        //
        // // update metadata
        // metadata = globals.metadata;
        return this.parsed;
    }

    private static prepareForParse(text: string): string {

        // This lets us use ¨ trema as an escape char to avoid md5 hashes
        // The choice of character is arbitrary; anything that isn't
        // magic in Markdown will work.
        text = text.replace(/¨/g, '¨T');

        // Replace $ with ¨D
        // RegExp interprets $ as a special character
        // when it's in a replacement string
        text = text.replace(/\$/g, '¨D');

        // Standardize line endings
        text = text.replace(/\r\n/g, '\n'); // DOS to Unix
        text = text.replace(/\r/g, '\n'); // Mac to Unix

        // Stardardize line spaces
        text = text.replace(/\u00A0/g, '&nbsp;');

        // Make sure text begins and ends with a couple of newlines:
        text = '\n\n' + text + '\n\n';

        return text;
    }
}